@@ -1,4 +1,3 @@
-import os
 import re
 import sys
 import time
@@ -22,21 +21,22 @@ def run_job(
     silent: bool = False,
     local: bool = False,
     target_name: str | None = None,
+    cpu: bool = False,
 ) -> None:
     try:
         gpu_idxs = None
         gpu_info = ""
 
-        if gpu_idxs_str:
+        if cpu:
+            num_gpus = 0
+            gpu_idxs = None
+            gpu_info = f" on {colored('CPU', 'cyan')}"
+        elif gpu_idxs_str:
             gpu_idxs = utils.parse_gpu_list(gpu_idxs_str)
             gpu_info = f" on GPU(s): {colored(','.join(map(str, gpu_idxs)), 'cyan')}"
         elif num_gpus:
             gpu_info = f" using {colored(str(num_gpus), 'cyan')} GPU(s)"
 
-        health = api_client.get_detailed_health(refresh=False, target_name=target_name)
-        if health.get("status") == "unhealthy":
-            utils.print_health_warning()
-
         if interactive:
             command = "bash"  # Use bash for interactive mode
             print(f"\n{colored('Starting interactive session:', 'blue', attrs=['bold'])}")
@@ -74,14 +74,22 @@ def run_job(
         git_ctx = None
         try:
             git_ctx = utils.prepare_git_artifact(cfg.enable_git_tag_push and not local, target_name=target_name)
-            env_vars = setup.load_current_env()
-            job_env_vars = dict(env_vars)
+            global_env = setup.load_current_env()
+            local_env = setup.load_local_env()
+            job_env_vars, conflicts = setup.merge_env_with_conflicts(global_env, local_env)
+
+            if local_env:
+                print(colored(f"\nLoaded {len(local_env)} variable(s) from local .env file", "cyan"))
+            if conflicts:
+                print(colored(f"\nLocal .env overriding {len(conflicts)} global variable(s):", "yellow"))
+                for key in conflicts.keys():
+                    print(f"  {colored('•', 'yellow')} {key}")
 
             invalid_notifications = []
 
             for notification_type in notifications:
                 required_vars = config.REQUIRED_ENV_VARS.get(notification_type, [])
-                if any(env_vars.get(var) is None for var in required_vars):
+                if any(job_env_vars.get(var) is None for var in required_vars):
                     invalid_notifications.append(notification_type)
 
             if invalid_notifications:
@@ -141,6 +149,10 @@ def run_job(
                 time.sleep(1)
                 try:
                     job = api_client.get_job(job_id, target_name=target_name)
+                    if job["status"] in ["failed", "killed", "completed"]:
+                        print(colored(f"\nJob {job_id} {job['status']}", "red" if job["status"] != "completed" else "green"))
+                        view_logs(cfg, target=job_id, target_name=target_name)
+                        return
                     if job["status"] == "running" and job.get("screen_session_name"):
                         print(colored(f"Job {job_id} running, attaching to screen session...", "green"))
                         attach_to_job(cfg, job_id, target_name=target_name)
@@ -180,6 +192,7 @@ def add_jobs(
     silent: bool = False,
     local: bool = False,
     target_name: str | None = None,
+    cpu: bool = False,
 ) -> None:
     try:
         if not commands:
@@ -192,21 +205,22 @@ def add_jobs(
         commands_list = [command_str]
 
         gpu_idxs = None
-        if gpu_idxs_str:
+        if cpu:
+            num_gpus = 0
+            gpu_idxs = None
+        elif gpu_idxs_str:
             gpu_idxs = utils.parse_gpu_list(gpu_idxs_str)
 
         expanded_commands = utils.expand_job_commands(commands_list, repeat=repeat)
         if not expanded_commands:
             return
 
-        health = api_client.get_detailed_health(refresh=False, target_name=target_name)
-        if health.get("status") == "unhealthy":
-            utils.print_health_warning()
-
         print(f"\n{colored('Adding the following jobs:', 'blue', attrs=['bold'])}")
         for cmd in expanded_commands:
             priority_str = f" (Priority: {colored(str(priority), 'cyan')})" if priority != 0 else ""
-            if gpu_idxs:
+            if cpu:
+                gpus_str = " (CPU)"
+            elif gpu_idxs:
                 gpus_str = f" (GPUs: {colored(','.join(map(str, gpu_idxs)), 'cyan')})"
             elif num_gpus > 1:
                 gpus_str = f" (GPUs: {colored(str(num_gpus), 'cyan')})"
@@ -236,7 +250,17 @@ def add_jobs(
                 if integration_type not in integrations:
                     integrations.append(integration_type)
 
-        env_vars = setup.load_current_env()
+        global_env = setup.load_current_env()
+        local_env = setup.load_local_env()
+        env_vars, conflicts = setup.merge_env_with_conflicts(global_env, local_env)
+
+        if local_env:
+            print(colored(f"\nLoaded {len(local_env)} variable(s) from local .env file", "cyan"))
+        if conflicts:
+            print(colored(f"\nLocal .env overriding {len(conflicts)} global variable(s):", "yellow"))
+            for key in conflicts.keys():
+                print(f"  {colored('•', 'yellow')} {key}")
+
         invalid_notifications = []
 
         for notification_type in notifications:
@@ -384,10 +408,10 @@ def show_history(regex: str | None = None, target_name: str | None = None) -> No
                 return job["created_at"]
             return 0
 
-        jobs.sort(key=get_sort_timestamp, reverse=False)
+        jobs.sort(key=get_sort_timestamp, reverse=True)
 
         print(colored("Job History:", "blue", attrs=["bold"]))
-        for job in jobs[:25]:
+        for job in reversed(jobs[:25]):
             runtime = utils.calculate_runtime(job)
             started_time = utils.format_timestamp(job.get("started_at"))
             status_color = (
@@ -690,8 +714,11 @@ def remove_jobs(job_ids: list[str], bypass_confirm: bool = False, target_name: s
         print(colored(f"Error removing jobs: {e}", "red"))
 
 
-def view_logs(target: str | None = None, tail: int | None = None, target_name: str | None = None) -> None:
+def view_logs(
+    cfg: config.NexusCliConfig, target: str | None = None, tail: int | None = None, target_name: str | None = None
+) -> None:
     try:
+        user = cfg.user or "anonymous"
         job_id: str = ""
         if target is None:
             jobs = []
@@ -702,11 +729,14 @@ def view_logs(target: str | None = None, tail: int | None = None, target_name: s
                 print(colored("No jobs found", "yellow"))
                 return
 
-            # Filter out jobs without a started_at timestamp and provide a default
-            # This prevents NoneType comparison errors during sorting
-            valid_jobs = [job for job in jobs if job.get("started_at") is not None]
+            user_jobs = [j for j in jobs if j.get("user") == user]
+            if not user_jobs:
+                print(colored(f"No jobs found for user '{user}'", "yellow"))
+                return
+
+            valid_jobs = [job for job in user_jobs if job.get("started_at") is not None]
             if not valid_jobs:
-                print(colored("No jobs with valid start times found", "yellow"))
+                print(colored(f"No jobs with valid start times found for user '{user}'", "yellow"))
                 return
 
             valid_jobs.sort(key=lambda x: x.get("started_at", 0), reverse=True)
@@ -1034,12 +1064,9 @@ def print_status(target_name: str | None = None) -> None:
 
         server_version = status.get("server_version", "unknown")
         if server_version != VERSION:
-            print(
-                colored(
-                    f"WARNING: Nexus client version ({VERSION}) does not match "
-                    f"Nexus server version ({server_version}).",
-                    "yellow",
-                )
+            raise RuntimeError(
+                f"Version mismatch: Nexus client version ({VERSION}) does not match "
+                f"server version ({server_version}). Please ensure client and server are on the same version."
             )
 
         health = api_client.get_detailed_health(refresh=False, target_name=target_name)
@@ -1047,49 +1074,108 @@ def print_status(target_name: str | None = None) -> None:
             utils.print_health_warning()
 
         node_name = status.get("node_name", "unknown")
-        queued = status.get("queued_jobs", 0)
-        running = status.get("running_jobs", 0)
         completed = status.get("completed_jobs", 0)
 
-        print(f"Node: {colored(node_name, 'cyan')}")
-        print(f"Queue: {queued} jobs pending")
-        print(f"Running: {running} jobs in progress")
-        print(f"History: {colored(str(completed), 'blue')} jobs completed\n")
-
         gpus = api_client.get_gpus(target_name=target_name)
+        running_jobs = api_client.get_jobs(status="running", target_name=target_name)
+        queued_jobs = api_client.get_jobs(status="queued", target_name=target_name)
 
-        print(colored("GPUs:", "white"))
-        for gpu in gpus:
-            memory_used = gpu.get("memory_used", 0)
-            gpu_info = f"GPU {gpu['index']} ({gpu['name']}): [{memory_used}/{gpu['memory_total']}MB] "
+        print(f"Node: {colored(node_name, 'cyan')}\n")
 
-            if gpu.get("is_blacklisted"):
-                gpu_info += colored("[BLACKLISTED] ", "red", attrs=["bold"])
+        available_gpus_list = [
+            str(g["index"])
+            for g in gpus
+            if not g.get("running_job_id") and not g.get("is_blacklisted") and g.get("process_count", 0) == 0
+        ]
+        in_use_gpus = [str(g["index"]) for g in gpus if g.get("running_job_id")]
+        external_gpus = [
+            str(g["index"])
+            for g in gpus
+            if not g.get("running_job_id") and not g.get("is_blacklisted") and g.get("process_count", 0) > 0
+        ]
+        blacklisted_gpus_list = [str(g["index"]) for g in gpus if g.get("is_blacklisted")]
 
-            if gpu.get("running_job_id"):
-                job_id = gpu["running_job_id"]
+        gpu_status_parts = []
+        if available_gpus_list:
+            count = len(available_gpus_list)
+            gpu_status_parts.append(f"{count} available {colored('[' + ', '.join(available_gpus_list) + ']', 'green')}")
+        if in_use_gpus:
+            count = len(in_use_gpus)
+            gpu_status_parts.append(f"{count} in use {colored('[' + ', '.join(in_use_gpus) + ']', 'cyan')}")
+        if external_gpus:
+            count = len(external_gpus)
+            gpu_status_parts.append(f"{count} external {colored('[' + ', '.join(external_gpus) + ']', 'yellow')}")
+        if blacklisted_gpus_list:
+            count = len(blacklisted_gpus_list)
+            gpu_status_parts.append(
+                f"{count} blacklisted {colored('[' + ', '.join(blacklisted_gpus_list) + ']', 'red')}"
+            )
 
-                job = api_client.get_job(job_id, target_name=target_name)
+        if gpu_status_parts:
+            print(f"{colored('GPUs:', 'white', attrs=['bold'])} {' | '.join(gpu_status_parts)}\n")
+        else:
+            print(f"{colored('GPUs:', 'white', attrs=['bold'])} {colored('None', 'yellow')}\n")
 
+        if running_jobs:
+            print(colored(f"Running Jobs ({len(running_jobs)}):", "white", attrs=["bold"]))
+            for job in sorted(running_jobs, key=lambda j: j.get("started_at", 0)):
                 runtime = utils.calculate_runtime(job)
                 runtime_str = utils.format_runtime(runtime)
-                start_time = utils.format_timestamp(job.get("started_at"))
+                utils.format_timestamp(job.get("started_at"))
 
-                job_gpu_count = job.get("num_gpus", 1)
-                gpu_count_str = f" ({job_gpu_count} GPUs)" if job_gpu_count > 1 else ""
+                if job.get("num_gpus", 0) == 0:
+                    resource_str = colored("CPU", "cyan")
+                else:
+                    gpu_idxs = job.get("gpu_idxs", [])
+                    if gpu_idxs:
+                        gpu_list = ",".join(map(str, gpu_idxs))
+                        resource_str = colored(f"GPU{'s' if len(gpu_idxs) > 1 else ''}: {gpu_list}", "cyan")
+                    else:
+                        resource_str = colored(
+                            f"{job.get('num_gpus')} GPU{'s' if job.get('num_gpus', 0) > 1 else ''}", "cyan"
+                        )
 
-                print(f"{gpu_info}{colored(job_id, 'magenta')}{gpu_count_str}")
-                print(f"  Command: {colored(job.get('command', ''), 'white', attrs=['bold'])}")
-                print(f"  Time: {colored(runtime_str, 'cyan')} (Started: {colored(start_time, 'cyan')})")
+                command = job.get("command", "")
+                if len(command) > 80:
+                    command = command[:77] + "..."
+
+                print(
+                    f"  {colored('•', 'white')} {colored(job['id'], 'magenta')} ({resource_str}) - {colored(runtime_str, 'cyan')}"
+                )
+                print(f"    {colored(command, 'white', attrs=['bold'])}")
                 if job.get("wandb_url"):
-                    print(f"  W&B: {colored(job['wandb_url'], 'yellow')}")
+                    print(f"    W&B: {colored(job['wandb_url'], 'yellow')}")
+                else:
+                    print()
+            print()
 
-            elif gpu.get("is_blacklisted", False):
-                print(f"{gpu_info}{colored('Blacklisted', 'red', attrs=['bold'])}")
-            elif gpu.get("process_count", 0) > 0:
-                print(f"{gpu_info}{colored('In Use (External Process)', 'yellow', attrs=['bold'])}")
-            else:
-                print(f"{gpu_info}{colored('Available', 'green', attrs=['bold'])}")
+        if queued_jobs:
+            preview_count = min(3, len(queued_jobs))
+            print(
+                colored(
+                    f"Queue ({len(queued_jobs)} job{'s' if len(queued_jobs) != 1 else ''} waiting):",
+                    "white",
+                    attrs=["bold"],
+                )
+            )
+            for idx, job in enumerate(
+                sorted(queued_jobs, key=lambda j: (-j.get("priority", 0), j.get("created_at", 0)))[:preview_count], 1
+            ):
+                if job.get("num_gpus", 0) == 0:
+                    resource_str = "CPU"
+                else:
+                    gpu_count = job.get("num_gpus", 1)
+                    resource_str = f"{gpu_count} GPU{'s' if gpu_count > 1 else ''}"
+
+                priority = job.get("priority", 0)
+                command = job.get("command", "")
+                if len(command) > 60:
+                    command = command[:57] + "..."
+
+                print(f"  {idx}. {colored(job['id'], 'magenta')} ({resource_str}, Priority: {priority}) - {command}")
+            print()
+
+        print(f"History: {colored(str(completed), 'blue')} job{'s' if completed != 1 else ''} completed")
 
     except Exception as e:
         print(colored(f"Error: {e}", "red"))
@@ -1157,33 +1243,45 @@ def attach_to_job(cfg: config.NexusCliConfig, target: str | None = None, target_
         job_id = job["id"]
         starting_status = job["status"]
 
+        import subprocess
+
+        import os as os_module
+
         active_target_name, target_cfg = config.get_active_target(target_name)
         if target_cfg is not None:
-            import subprocess
-
-            ssh_key = config.get_ssh_key_path(target_cfg.host, target_cfg.port)
-            if not ssh_key.exists():
-                print(colored(f"SSH key not found at {ssh_key}", "red"))
-                print(colored("Run 'nx targets add' to configure remote access", "yellow"))
-                return
-
-            import os as os_module
-
             env = os_module.environ.copy()
             env["TERM"] = "xterm-256color"
 
+            subprocess.call(
+                [
+                    "ssh",
+                    "-o",
+                    "StrictHostKeyChecking=accept-new",
+                    f"{target_cfg.ssh_user}@{target_cfg.host}",
+                    "sudo",
+                    "-u",
+                    "nexus",
+                    "screen",
+                    "-S",
+                    screen_session_name,
+                    "-X",
+                    "acladd",
+                    target_cfg.ssh_user,
+                ],
+                stdout=subprocess.DEVNULL,
+                stderr=subprocess.DEVNULL,
+            )
+
             exit_code = subprocess.call(
                 [
                     "ssh",
                     "-t",
-                    "-i",
-                    str(ssh_key),
                     "-o",
                     "StrictHostKeyChecking=accept-new",
-                    f"nexus@{target_cfg.host}",
+                    f"{target_cfg.ssh_user}@{target_cfg.host}",
                     "screen",
-                    "-r",
-                    screen_session_name,
+                    "-x",
+                    f"nexus/{screen_session_name}",
                 ],
                 env=env,
             )
@@ -1193,14 +1291,14 @@ def attach_to_job(cfg: config.NexusCliConfig, target: str | None = None, target_
                 print(colored(f"View logs: nx logs {job_id}", "yellow"))
                 return
         else:
-            current_user_exit_code = os.system(f"screen -r {screen_session_name}")
+            current_user_exit_code = subprocess.call(["screen", "-x", f"nexus/{screen_session_name}"])
 
             if current_user_exit_code != 0:
-                exit_code = os.system(f"sudo -u nexus screen -r {screen_session_name}")
+                exit_code = subprocess.call(["sudo", "-u", "nexus", "screen", "-r", screen_session_name])
 
                 if exit_code != 0:
                     print(colored("Screen session not found. Available sessions:", "yellow"))
-                    os.system("screen -ls")
+                    subprocess.call(["screen", "-ls"])
                     print(colored("\nTroubleshooting tips:", "yellow"))
                     print("  1. Verify that the job is still running and the session name is correct.")
                     print("  2. Check if you have the proper permissions to access the screen session.")
