@@ -91,30 +91,38 @@ def save_working_state() -> tuple[str, str, str, bool]:
     try:
         original_branch = get_current_git_branch()
 
+        try:
+            original_head = subprocess.check_output(
+                ["git", "rev-parse", "HEAD"], text=True
+            ).strip()
+        except subprocess.CalledProcessError:
+            raise RuntimeError(
+                "Cannot create artifact from repository with no commits. "
+                "Please make an initial commit first."
+            )
+
         is_dirty = is_working_tree_dirty()
-        if is_dirty:
-            subprocess.run(["git", "stash", "-u"], check=True, capture_output=True)
+        if not is_dirty:
+            tree_sha = subprocess.check_output(["git", "rev-parse", "HEAD^{tree}"], text=True).strip()
+            return (original_branch, "", tree_sha, False)
 
+        subprocess.run(["git", "add", "-A"], check=True, capture_output=True)
+        subprocess.run(["git", "commit", "--no-verify", "-m", "Nexus temporary commit"], check=True, capture_output=True)
+
+        tree_sha = subprocess.check_output(["git", "rev-parse", "HEAD^{tree}"], text=True).strip()
         temp_branch = f"nexus-tmp-{int(time.time())}-{generate_job_id()}"
-        subprocess.run(["git", "checkout", "-b", temp_branch], check=True, capture_output=True)
+        subprocess.run(["git", "branch", temp_branch], check=True, capture_output=True)
 
-        if is_dirty:
-            subprocess.run(["git", "stash", "apply"], check=True, capture_output=True)
-            subprocess.run(["git", "add", "-A"], check=True, capture_output=True)
-            subprocess.run(["git", "commit", "-m", "Nexus temporary commit"], check=True, capture_output=True)
+        subprocess.run(["git", "reset", original_head], check=True, capture_output=True)
 
-        commit_sha = subprocess.check_output(["git", "rev-parse", "HEAD"], text=True).strip()
-
-        return (original_branch, temp_branch, commit_sha, is_dirty)
+        return (original_branch, temp_branch, tree_sha, True)
     except subprocess.CalledProcessError as e:
         raise RuntimeError(f"Failed to save working state: {e}")
 
 
 def restore_working_state(original_branch: str, temp_branch: str, we_created_stash: bool) -> None:
-    subprocess.run(["git", "checkout", original_branch], check=True, capture_output=True)
-    if we_created_stash:
-        subprocess.run(["git", "stash", "pop"], check=True, capture_output=True)
-    subprocess.run(["git", "branch", "-D", temp_branch], check=True, capture_output=True)
+    if temp_branch:
+        subprocess.run(["git", "branch", "-D", temp_branch], check=True, capture_output=True)
 
 
 def prepare_git_artifact(enable_git_tag_push: bool, target_name: str | None = None) -> GitArtifactContext:
@@ -138,38 +146,54 @@ def prepare_git_artifact(enable_git_tag_push: bool, target_name: str | None = No
     commit_sha = None
     we_created_stash = False
 
-    if is_working_tree_dirty():
-        original_branch, temp_branch, commit_sha, we_created_stash = save_working_state()
-
-    result = subprocess.run(["git", "config", "--get", "remote.origin.url"], capture_output=True, text=True)
-    git_repo_url = result.stdout.strip() or "unknown-url"
-
     try:
-        print(colored("Creating git archive...", "blue"))
-        artifact_data = create_git_archive(temp_branch or "HEAD")
+        if is_working_tree_dirty():
+            original_branch, temp_branch, commit_sha, we_created_stash = save_working_state()
+        else:
+            result = subprocess.run(["git", "rev-parse", "HEAD^{tree}"], capture_output=True, text=True, check=True)
+            commit_sha = result.stdout.strip()
 
+        result = subprocess.run(["git", "config", "--get", "remote.origin.url"], capture_output=True, text=True)
+        git_repo_url = result.stdout.strip() or "unknown-url"
+
+        artifact_id = None
         git_tag = None
-        if enable_git_tag_push and can_push_to_remote("origin"):
-            tag_name = f"nexus-{job_id}"
-            if commit_sha:
-                ensure_git_tag(tag_name, message=f"Nexus job {job_id}", commit_ref=commit_sha)
-            else:
-                ensure_git_tag(tag_name, message=f"Nexus job {job_id}")
+        if commit_sha:
+            print(colored(f"Checking for existing artifact (tree: {commit_sha[:8]})...", "blue"))
+            exists, existing_artifact_id = api_client.check_artifact_by_sha(commit_sha, target_name=target_name)
+            if exists and existing_artifact_id:
+                print(colored(f"Reusing existing artifact {existing_artifact_id}", "green"))
+                artifact_id = existing_artifact_id
 
-            try:
-                push_git_tag(tag_name, remote="origin")
-                print(colored(f"Pushed git tag: {tag_name}", "green"))
-                git_tag = tag_name
-            except RuntimeError as e:
-                print()
-                print(colored(f"ERROR: Failed to push git tag {tag_name}", "red", attrs=["bold"]))
-                print(colored(f"       {e}", "red"))
-                print(colored("       Common fixes:", "red"))
-                print(colored("       • Check git authentication (SSH keys or credentials)", "red"))
-                print(colored("       • Verify network connectivity to remote", "red"))
-                print(colored("       • Ensure you have push permissions to the repository", "red"))
-                print(colored("       Job will continue without git tag.", "red"))
-                print()
+        if artifact_id is None:
+            print(colored("Creating git archive...", "blue"))
+            artifact_data = create_git_archive(temp_branch or "HEAD")
+
+            if enable_git_tag_push and can_push_to_remote("origin"):
+                tag_name = f"nexus-{job_id}"
+                if commit_sha:
+                    ensure_git_tag(tag_name, message=f"Nexus job {job_id}", commit_ref=commit_sha)
+                else:
+                    ensure_git_tag(tag_name, message=f"Nexus job {job_id}")
+
+                try:
+                    push_git_tag(tag_name, remote="origin")
+                    print(colored(f"Pushed git tag: {tag_name}", "green"))
+                    git_tag = tag_name
+                except RuntimeError as e:
+                    print()
+                    print(colored(f"ERROR: Failed to push git tag {tag_name}", "red", attrs=["bold"]))
+                    print(colored(f"       {e}", "red"))
+                    print(colored("       Common fixes:", "red"))
+                    print(colored("       • Check git authentication (SSH keys or credentials)", "red"))
+                    print(colored("       • Verify network connectivity to remote", "red"))
+                    print(colored("       • Ensure you have push permissions to the repository", "red"))
+                    print(colored("       Job will continue without git tag.", "red"))
+                    print()
+
+            print(colored("Uploading git archive...", "blue"))
+            artifact_id = api_client.upload_artifact(artifact_data, git_sha=commit_sha, target_name=target_name)
+            print(colored(f"Artifact uploaded with ID: {artifact_id}", "green"))
 
         if temp_branch and original_branch:
             restore_working_state(original_branch, temp_branch, we_created_stash)
@@ -179,10 +203,6 @@ def prepare_git_artifact(enable_git_tag_push: bool, target_name: str | None = No
             temp_branch_saved = temp_branch
             original_branch_saved = original_branch
 
-        print(colored("Uploading git archive...", "blue"))
-        artifact_id = api_client.upload_artifact(artifact_data, target_name=target_name)
-        print(colored(f"Artifact uploaded with ID: {artifact_id}", "green"))
-
         return GitArtifactContext(
             job_id=job_id,
             artifact_id=artifact_id,
@@ -194,10 +214,11 @@ def prepare_git_artifact(enable_git_tag_push: bool, target_name: str | None = No
             we_created_stash=we_created_stash,
             git_tag=git_tag,
         )
-    except Exception:
+    finally:
         if temp_branch and original_branch:
-            restore_working_state(original_branch, temp_branch, we_created_stash)
-        raise
+            current = get_current_git_branch()
+            if current == temp_branch:
+                restore_working_state(original_branch, temp_branch, we_created_stash)
 
 
 def cleanup_git_state(ctx: GitArtifactContext) -> None:
