@@ -1,5 +1,4 @@
 import pathlib as pl
-import socket
 
 import uvicorn
 
@@ -13,26 +12,15 @@ def _run_server(server_dir: pl.Path | None) -> None:
     ctx = setup.initialize_context(server_dir)
     api_app = app.create_app(ctx)
 
-    has_ssl = ctx.config.ssl_keyfile and ctx.config.ssl_certfile
-
-    if has_ssl:
-        print("🔒 SSL: enabled")
-        print("🔑 Token: configured")
-        print(f"🌐 https://{socket.gethostname()}:{ctx.config.port} (0.0.0.0:{ctx.config.port})")
-        print()
-    elif ctx.config.server_dir is not None:
-        print("⚠️  No API token - LOCAL ONLY")
-        print("\nTo enable remote: Install server with nexus-server install")
-        print(f"\nServer: http://localhost:{ctx.config.port}\n")
-    else:
-        setup.display_config(ctx.config)
+    print(f"🖥️  Node: {ctx.config.node_name}")
+    print("🔒 Auth: SSH tunnel (remote access via SSH port forwarding)")
+    print(f"🌐 Server: http://127.0.0.1:{ctx.config.port} (localhost only)")
+    print()
 
     uvicorn.run(
         api_app,
-        host="0.0.0.0" if has_ssl else "localhost",
+        host="127.0.0.1",
         port=ctx.config.port,
-        ssl_keyfile=ctx.config.ssl_keyfile,
-        ssl_certfile=ctx.config.ssl_certfile,
     )
 
 
